import _extends from "@babel/runtime/helpers/esm/extends";
import { Children, cloneElement, createRef, forwardRef, useCallback, useEffect, useRef, useState } from 'react';
import PropTypes from 'prop-types';
import { getTarget } from '@s-ui/js/lib/react';
import { inputSizes as SELECT_SIZES } from '@s-ui/react-atom-input';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import { moleculeDropdownListSizes as SIZES } from '@s-ui/react-molecule-dropdown-list';
import MoleculeSelectMultipleSelection from './components/MultipleSelection.js';
import MoleculeSelectSingleSelection from './components/SingleSelection.js';
import { ENABLED_KEYS, getClassName, getOptionData, SELECT_STATES, SELECTION_KEYS } from './config.js';
import { jsx as _jsx } from "react/jsx-runtime";
var MoleculeSelect = /*#__PURE__*/forwardRef(function (props, forwardedRef) {
  var onBlur = props.onBlur,
    isOpen = props.isOpen,
    onToggle = props.onToggle,
    children = props.children,
    errorState = props.errorState,
    state = props.state,
    disabled = props.disabled,
    keysSelection = props.keysSelection,
    multiselection = props.multiselection,
    refMoleculeSelectFromProps = props.refMoleculeSelect,
    ariaLabel = props['aria-label'];
  var refMoleculeSelect = useRef(refMoleculeSelectFromProps);
  var refsMoleculeSelectOptions = useRef([]);
  var ref = useMergeRefs(forwardedRef, refMoleculeSelect);
  var _useState = useState(isOpen),
    isOpenState = _useState[0],
    setIsOpenState = _useState[1];
  useEffect(function () {
    return setIsOpenState(isOpen);
  }, [isOpen, setIsOpenState]);
  var _useState2 = useState(getOptionData(children)),
    optionsData = _useState2[0],
    setOptionsData = _useState2[1];
  var _useState3 = useState(false),
    focus = _useState3[0],
    setFocus = _useState3[1];
  var options = refsMoleculeSelectOptions.current.map(getTarget);
  var extendedChildren = Children.toArray(children).filter(Boolean).map(function (child, index) {
    refsMoleculeSelectOptions.current[index] = /*#__PURE__*/createRef();
    return /*#__PURE__*/cloneElement(child, {
      innerRef: refsMoleculeSelectOptions.current[index],
      selectKey: keysSelection
    });
  });
  var className = getClassName({
    state: state,
    errorState: errorState,
    focus: focus,
    disabled: disabled
  });
  var closeList = useCallback(function (ev, _ref) {
    var _ref$isOutsideEvent = _ref.isOutsideEvent,
      isOutsideEvent = _ref$isOutsideEvent === void 0 ? false : _ref$isOutsideEvent;
    setIsOpenState(false);
    if (!isOutsideEvent) {
      ev.preventDefault();
      ev.stopPropagation();
    }
  }, []);
  var handleOutsideClick = useCallback(function (ev) {
    if (disabled) return;
    if (refMoleculeSelect.current && !refMoleculeSelect.current.contains(ev.target)) {
      // outside click
      closeList(ev, {
        isOutsideEvent: true
      });
      setFocus(false);
    }
  }, [closeList, disabled]);
  useEffect(function () {
    setOptionsData(getOptionData(children));
    document.addEventListener('touchend', handleOutsideClick);
    document.addEventListener('mousedown', handleOutsideClick);
    return function () {
      document.removeEventListener('touchend', handleOutsideClick);
      document.removeEventListener('mousedown', handleOutsideClick);
    };
  }, [children, handleOutsideClick]);
  var focusFirstOption = function focusFirstOption(ev) {
    options[0] && options[0].focus();
    ev.preventDefault();
    ev.stopPropagation();
  };
  var handleToggle = function handleToggle(ev, _temp) {
    var _ref2 = _temp === void 0 ? {} : _temp,
      isOpen = _ref2.isOpen;
    setIsOpenState(isOpen !== undefined ? isOpen : !isOpenState);
    typeof onToggle === 'function' && onToggle(ev, {
      isOpen: isOpen
    });
    ev.preventDefault();
    ev.stopPropagation();
  };
  var handleKeyDown = function handleKeyDown(ev) {
    ev.persist();
    var isEnabledKey = ENABLED_KEYS.includes(ev.key);
    var domSourceEvent = ev.target;
    var domMoleculeSelect = refMoleculeSelect.current;
    if (!isOpenState && isEnabledKey) {
      domSourceEvent === domMoleculeSelect && setIsOpenState(!isOpenState);
      setTimeout(function () {
        return focusFirstOption(ev);
      });
    } else if (ev.key === 'Escape') {
      closeList(ev, {
        isOutsideEvent: true
      });
    } else {
      setTimeout(function () {
        return focusFirstOption(ev);
      });
    }
  };
  var handleFocusOut = function handleFocusOut(event) {
    onBlur(event);
    setFocus(false);
  };
  var handleFocusIn = function handleFocusIn() {
    return !disabled && setFocus(true);
  };
  var handleClick = function handleClick(ev) {
    ev.persist();
    if (focus) {
      setTimeout(function () {
        return focusFirstOption(ev);
      });
    }
  };
  var Select = multiselection ? MoleculeSelectMultipleSelection : MoleculeSelectSingleSelection;
  return /*#__PURE__*/_jsx("div", {
    ref: ref,
    tabIndex: "0",
    className: className,
    onKeyDown: handleKeyDown,
    onFocus: handleFocusIn,
    onBlur: handleFocusOut,
    onClick: handleClick,
    "aria-label": ariaLabel,
    children: /*#__PURE__*/_jsx(Select, _extends({
      refMoleculeSelect: refMoleculeSelect,
      optionsData: optionsData,
      isOpen: isOpenState,
      onToggle: handleToggle
    }, props, {
      children: extendedChildren
    }))
  });
});
MoleculeSelect.defaultProps = {
  onBlur: function onBlur() {},
  disabled: false,
  keysSelection: SELECTION_KEYS,
  onChange: function onChange() {},
  readOnly: false,
  selectSize: SELECT_SIZES.MEDIUM
};
MoleculeSelect.displayName = 'MoleculeSelect';
export default MoleculeSelect;
export { SIZES as moleculeSelectDropdownListSizes };
export { SELECT_SIZES as moleculeSelectSizes };
export { SELECT_STATES as moleculeSelectStates };